/* *******************************************************************
   ESP8266 Extended Client

   by noiasca:

   Hardware
   - NodeMCU / ESP8266

   Short
   - webclient only
   - webclient sends data to another webserver
   - webclient receives data from another webserver
   - ArduinoOTA Upload with Arduino IDE
   - open serial monitor and send a g to test client

   Version
   2023-06-30   ESP8266 core 2.7.4
   2020-06-08   ESP8266 core 2.5.0 work in pogress

***************************************************************** */
#include <ESP8266WiFi.h>
#include <WiFiClient.h>
#include <ESP8266HTTPClient.h>                             // for the webclient https://github.com/esp8266/Arduino/tree/master/libraries/ESP8266HTTPClient
#include <ESP8266mDNS.h>                                   // Bonjour/multicast DNS ... find the device on network by name
#include <ArduinoOTA.h>                                    // OTA Upload via ArduinoIDE
                                                           
#define VERSION "9.0.0"                                    // this is just the version of the Arduino Sketch
                                                           
#define USE_BOARD 2                                        // the actual board to compile

/* *******************************************************************
         the board settings / die Einstellungen der verschiedenen Boards
 ********************************************************************/

#if USE_BOARD == 2                                         // Example Board
#define TXT_BOARDID "2"                                    // an ID for the board
#define TXT_BOARDNAME "ESP Extended Client"                // The name of the board
const uint16_t clientIntervall = 30;                       // intervall to send data to a server in seconds. Set to 0 if you don't want to send data
const char* sendHttpTo = "http://172.18.67.124/data";      // the module will send information to that server/resource. Use an URI or a IP Adress
#endif

/* *******************************************************************
         other settings / weitere Einstellungen für den Anwender
 ********************************************************************/

//#include <credentials.h>                                   // if you have a config file with your credentials

#ifndef STASSID
#define STASSID "your-ssid"                                // set your SSID
#define STAPSK  "your-password"                            // set your wifi password
#endif

const char* ssid =     STASSID;
const char* password = STAPSK;

/* *******************************************************************
         Globals - Variables and constants
 ********************************************************************/

unsigned long ss = 0;                                      // current second since startup
uint32_t clientPreviousSs = 0 - clientIntervall;           // last second when data was sent to server

const uint8_t button1Pin = 0;         // GPIO00/D3 on NodeMCU is the Flash Button - use this for testing an input
const uint8_t output1Pin = 16;        // GPIO16/D0 on NodeMCU is a (red) LED on the NodeMCU Board - use this for testing the html switch
const uint8_t output2Pin = 2;         // GPIO02/D4 on NodeMCU is the (blue) LED on the ESP-12E

ADC_MODE(ADC_VCC);                     // to use getVcc
uint32_t internalVcc = ESP.getVcc();

//values from the remote device will be stored in this variables:
uint8_t remoteBoardId = 0;                       // will save the remote board ID
uint8_t remoteButton1 = 0;                       // the input GPIO from the remote ESP
uint8_t remoteOutput1 = 0;                       // the output GPIO from the remote ESP
uint8_t remoteOutput2 = 0;                       // the output GPIO from the remote ESP
uint32_t remoteVcc = 0;                          // the voltage measured by remote ESP
float remoteFloat = 0;
uint32_t remoteMessagesSucessfull = 0;           // counter, how many messages where received
uint32_t remoteLastMessage = 0;                  // last message from remote ESP

/* *******************************************************************
         S E T U P
 ********************************************************************/

void setup(void) {
  Serial.begin(115200);
  pinMode(button1Pin, INPUT);
  pinMode(output1Pin, OUTPUT);
  pinMode(output2Pin, OUTPUT);
  delay(100); // the serial needs some time
  Serial.println(F("\n" TXT_BOARDNAME "\nVersion: " VERSION " Board " TXT_BOARDID " "));
  Serial.print  (__DATE__);
  Serial.print  (' ');
  Serial.println(__TIME__);

  char myhostname[8] = {"esp"};
  strcat(myhostname, TXT_BOARDID);
  WiFi.hostname(myhostname);
  WiFi.begin(ssid, password);
  // Wait for connection
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print('.');
  }
  Serial.print(F("\n\nConnected to "));
  Serial.println(ssid);
  Serial.print(F("IP address: "));
  Serial.println(WiFi.localIP());
  if (MDNS.begin(myhostname)) {        // also for the OTA Upload of the ESP for the Arduino IDE
    Serial.println(F("MDNS responder started"));
  }

  //IDE OTA
  ArduinoOTA.setHostname(myhostname);  // give a name to your ESP for the Arduino IDE
  ArduinoOTA.begin();                  // OTA Upload via ArduinoIDE https://arduino-esp8266.readthedocs.io/en/latest/ota_updates/readme.html
}

/* *******************************************************************
         M A I N L O O P
 ********************************************************************/

void loop(void) {
  ss = millis() / 1000;
  if (clientIntervall > 0 && (ss - clientPreviousSs) >= clientIntervall) {
    sendPost();
    clientPreviousSs = ss;
  }
  
  ArduinoOTA.handle();                 // OTA Upload via ArduinoIDE
  
  // you can use Serial to send imidiately;
  int c = Serial.read();
  if (c == 'g') sendPost();
  
  // do your sensor readings, calculations,  outputs to displays etc...

}
//
